﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.Web;
using System.Reflection;
using System.Collections;
using System.Web.Caching;
using Common.Ajax.Service;
using Common.Ajax.Extension;

namespace Common.Ajax.Utility
{
    /// <summary>
    /// 核心类，执行ajax请求
    /// </summary>
    internal static class Executor
    {
        /// <summary>
        /// Hashtable，缓存页面ajax方法委托
        /// </summary>
        private static Hashtable ajaxDelegateTable = Hashtable.Synchronized(new Hashtable());

        /// <summary>
        /// 核心函数，执行Handler的方法
        /// </summary>
        /// <param name="page">页面对象</param>
        /// <param name="context">请求上下文</param>
        /// <param name="cacheMethodInfo">缓存方法原数据</param>
        internal static void Execute(Page page, HttpContext context, CacheMethodInfo methodInfo)
        {
            if (page == null)
            {
                throw new ArgumentNullException("page");
            }
            try
            {
                if (methodInfo != null)
                {
                    HttpRequest request = context.Request;
                    object[] parameters = GetParametersFromRequest(request, methodInfo.Parameters);
                    object data = methodInfo.Func(page, parameters);
                    int serverCache = methodInfo.AjaxMethodAttribute.ServerCache;
                    if (serverCache > 0)
                    {
                        CacheHelper.Insert(context, methodInfo.AjaxMethodAttribute.ServerCache, data);
                    }
                    EndCurrentRequest(context, data, methodInfo.AjaxMethodAttribute.OutputCache);
                }
                else
                {
                    EndCurrentRequest(context, "找不到合适的Ajax方法！");
                }
            }
            catch (FormatException)
            {
                EndCurrentRequest(context, "调用方法匹配到无效的参数！");
            }
            catch (InvalidCastException)
            {
                EndCurrentRequest(context, "参数转换出错！");
            }
            catch (System.Threading.ThreadAbortException)
            {
                //do nothing
            }
            catch (Exception ex)
            {
                EndCurrentRequest(context, ex.Message);
            }
        }

        /// <summary>
        /// 获取页面标记方法信息
        /// </summary>
        /// <param name="page">页面对象</param>
        /// <param name="methodName">方法名称</param>
        internal static CacheMethodInfo GetDelegateInfo(Page page, string methodName)
        {
            if (page == null)
            {
                throw new ArgumentNullException("page");
            }
            Type type = page.GetType();
            Dictionary<string, CacheMethodInfo> dic = ajaxDelegateTable[type.AssemblyQualifiedName] as Dictionary<string, CacheMethodInfo>;
            if (dic == null)
            {
                dic = new Dictionary<string, CacheMethodInfo>();
                IEnumerable<CacheMethodInfo> infos = (from m in type.GetMethods(BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static)
                                                      let ca = m.GetCustomAttributes(typeof(AjaxMethodAttribute), false).FirstOrDefault()
                                                      where ca != null
                                                      select new CacheMethodInfo
                                                      {
                                                          AjaxMethodAttribute = ca as AjaxMethodAttribute,
                                                          MethodName = m.Name,
                                                          Parameters = m.GetParameters()
                                                      });

                if (infos.IsNullOrEmpty())
                {
                    return null;
                }                    
                for (int i = 0, length = infos.Count(); i < length; i++)
                {                    
                    CacheMethodInfo cacheMethodInfo = infos.ElementAt(i);                    
                    string name = cacheMethodInfo.MethodName;
                    MethodInfo methodInfo = type.GetMethod(name);
                    if (!dic.ContainsKey(name))
                    {                        
                        cacheMethodInfo.Func = ReflectionUtil.GetMethodDelegate(methodInfo);
                        dic.Add(name, cacheMethodInfo);
                    }
                }
                ajaxDelegateTable[type.AssemblyQualifiedName] = dic;
            }
            CacheMethodInfo currentMethodInfo = null;
            dic.TryGetValue(methodName, out currentMethodInfo);
            return currentMethodInfo;            
        }

        /// <summary>
        /// 从请求获取参参数
        /// </summary>
        /// <param name="request">HttpRequest</param>
        ///<param name="parameters">参数信息</param>
        /// <returns>参数数组</returns>
        private static object[] GetParametersFromRequest(HttpRequest request, ParameterInfo[] parameters)
        {
            if (parameters.IsNullOrEmpty())
            {
                return null;
            }
            int length = parameters.Length;
            object[] realParameters = new object[length];
            for (int i = 0; i < length; i++)
            {
                ParameterInfo pi = parameters[i];
                Type piType = pi.ParameterType.GetRealType();
                object value = null;
                if (piType.IsValueType())
                {
                    //值类型
                    value = ModelUtil.GetValue(request, pi.Name, piType);
                    value = value ?? Activator.CreateInstance(piType);
                }
                else if (piType.IsClass)
                {
                    //引用类型
                    object model = ModelUtil.CreateModel(piType);
                    ModelUtil.FillModelByRequest(request, pi.Name, piType, model);
                    value = model;
                }
                else
                {
                    throw new NotSupportedException(pi.Name + " 参数不被支持");
                }
                realParameters[i] = value;
            }
            return realParameters;
        }

        internal static void EndCurrentRequest(HttpContext context, object data)
        {
            EndRequest(context, data, 0, ContentType.Plain);
        }

        internal static void EndCurrentRequest(HttpContext context, object data, int outPutCache)
        {
            EndRequest(context, data, outPutCache, ContentType.Plain);
        }

        private static void EndRequest(HttpContext context, object data, int outPutCache, ContentType contentType)
        {
            HttpResponse response = context.Response;
            if (outPutCache != 0)
            {
                if (string.Compare(context.Request.HttpMethod, "GET", true) == 0)
                {
                    if (outPutCache > 0)
                    {
                        response.Cache.SetCacheability(HttpCacheability.Public);
                        response.Cache.SetMaxAge(new TimeSpan(0, 0, outPutCache));
                        response.Cache.SetExpires(DateTime.Now.AddSeconds(outPutCache));
                    }
                    else
                    {
                        response.Cache.SetCacheability(HttpCacheability.NoCache);
                        response.Cache.SetNoStore();
                    }
                }
            }
            response.ContentType = GetContentType(contentType);
            response.ContentEncoding = System.Text.Encoding.UTF8;
            if (data != null)
            {
                response.Write(data);
            }
            response.End();
        }

        private static string GetContentType(ContentType contentType)
        {
            switch (contentType)
            {
                case ContentType.Plain: return "text/plain";
                case ContentType.XML: return "text/xml";
                case ContentType.Html: return "text/html";
                case ContentType.JSON: return "application/json";
                case ContentType.Javascript: return "text/javascript";
                default: return "text/plain";
            }
        }
    }
}